/**
 * The purpose of this module is to expose a function which overrides the original paths to use server side paths
 * when applicable
 *
 * Author: Pubudu Dodangoda
 */

'use strict';

const immer = require('immer');

immer.setAutoFreeze(false);
const clone = immer.default;

module.exports = function(pathsBase, resolveApp, resolveModule) {
  return clone(pathsBase, paths => {
    const ssr = process.argv.some(arg => arg === '--ssr');
    if (ssr) {
      paths.appBuild = resolveApp('build/public');
      paths.nodeBuild = resolveApp('build');
      paths.nodeBuildAppJs = resolveApp('build/server.js');
      paths.appIndexJs = resolveModule(resolveApp, 'src/app/index');
      paths.appNodeJs = {
        dev: resolveModule(resolveApp, 'src/server/app'),
        prod: resolveModule(resolveApp, 'src/server/index'),
      };
      paths.appNodeJsOutput = {
        dev: 'server',
        prod: 'server',
      };
      paths.configOverrides = resolveApp('config-overrides.js');
      paths.assetsJson = resolveApp('build/assets.json');
    }

    return paths;
  });
};
