'use strict';

const path = require('path');
const paths = require('./paths');
const clone = require('immer').default;
const HtmlWebpackPlugin = require('html-webpack-plugin');
const InterpolateHtmlPlugin = require('react-dev-utils/InterpolateHtmlPlugin');
const InlineChunkHtmlPlugin = require('react-dev-utils/InlineChunkHtmlPlugin');
const TerserPlugin = require('terser-webpack-plugin');

const configFactory = require('./webpack.config');

module.exports = function(webpackEnv) {
  const configEnv = process.argv.some(arg => arg === '--writeToDisk')
    ? 'production'
    : webpackEnv;
  const config = configFactory(configEnv);

  const prodOverride = configEnv === 'production';
  const devOverride = webpackEnv === 'development';

  if (prodOverride) {
    return clone(config, config => {
      const noMinify = process.argv.some(arg => arg === '--no-minify');
      if (noMinify) {
        config.optimization.minimizer = config.optimization.minimizer.filter(
          item => {
            return !(item instanceof TerserPlugin);
          }
        );
      }

      const ssr = process.argv.some(arg => arg === '--ssr');
      if (ssr) {
        config.plugins = config.plugins.filter(plugin => {
          return (
            !(plugin instanceof HtmlWebpackPlugin) &&
            !(plugin instanceof InterpolateHtmlPlugin) &&
            !(plugin instanceof InlineChunkHtmlPlugin)
          );
        });
      }

      return config;
    });
  }
  if (devOverride) {
    return clone(config, config => {
      // Allow styled components based libraries to be used with npm link: Otherwise the theme will become buggy due to multiple instances of styled-components
      config.resolve.alias['styled-components'] = path.resolve(
        paths.appPath,
        'node_modules',
        'styled-components'
      );

      return config;
    });
  }
};
